import {Component, OnInit, ViewChild} from '@angular/core';
import { ActivatedRoute, NavigationStart, Router } from '@angular/router';
import {
  IgxGridComponent,
  IGridKeydownEventArgs,
  ϴIgxGridCellComponent,
  IActiveNodeChangeEventArgs
} from '@infragistics/igniteui-angular';
import { NgxSmartModalService } from 'ngx-smart-modal';
import { ApiService } from 'src/app/shared/services/api.service';
import { trim } from 'igniteui-angular-core';
import { ComboboxService } from 'src/app/shared/services/combobox.service';
import { getDataRow } from "./data-item";
import { validateFromSearchJfcalc } from '../validateSimulation';
import { EntityService } from "../../../shared/services/entity.service";
import { Def } from "../../../shared/entities/heatExchanger/baseHeatExchanger";
import { Subscription } from "rxjs";
import { Simulation } from "../../../shared/entities/simulation";
import { getHeadExchanger, RadiatorNames } from "../../../shared/entities/utils";
import {getDataHandleKeyDown} from "../../../shared/utils/get-data-handle-key-down";

@Component({
  selector: 'app-cond',
  templateUrl: './cond.component.html',
  styleUrls: ['./cond.component.scss']
})
export class CondComponent implements OnInit {
  subsc: Subscription = new Subscription();
  simulation: Simulation;
  widthTitle = '230px';
  arrWidthTitle = [
    '',
    '230px',
    '320px',
    '260px',
    '230px',
    '280px',
    '280px',
    '320px',
    '280px',
    '230px',
  ];

  @ViewChild('grid1', { static: true }) public grid1: IgxGridComponent;

  @ViewChild('grid2', { static: true }) public grid2: IgxGridComponent;

  loading = false;
  radiatorTypeName = '';

  getDataRow = {
    arrRequiredDeat: [],
    indexWithFinA: null,
    indexWithFinB: null,
    indexTubeType: null,
    arrChangeTubeType: [[], [], []],
    arrCheckboxWithFinA: [[], []],
    arrCheckboxWithFinB: [[], []],
  };

  cellSelection = [];
  cellSelected: any;
  combo2: any;
  combo3: any;
  combo9: any;
  combo10: any;
  combo12: any;

  columns = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16];

  rowData = [];

  arrTitleRowData = [];

  fluidTypeA = false;
  fluidTypeB = false;

  rowData2 = [];
  dataPdf = [];

  cellClsDirtyCheck = {
    'overflow-hide': true,
    'text-center': true,
    dirty: this.dirtyCheck,
  };

  cellCls = {
    'text-center': true,
  };

  cellCls2 = {
    'text-center': true,
    'text-th': true,
    'border-bottom-dcdbdc': true,
  };

  copyOptions = {
    enabled: true,
    copyHeaders: false,
    copyFormatters: false,
    separator: '\t'
  };

  showDetail = false;
  simulationBasicColumns: Def[];
  simulationDetailColumns: Def[];
  eventClick: boolean = false;

  constructor(private activatedRoute: ActivatedRoute, private router: Router, public entity: EntityService, public api: ApiService, public combobox: ComboboxService, public ngxSmartModalService: NgxSmartModalService) {
    this.subsc.add(this.entity.simulation$.subscribe(simulation => {
      this.simulation = simulation;

      if (!getHeadExchanger(this.activatedRoute.snapshot.params.type)) {
        this.router.navigate(['/heat-exchanger/simulation/select']);
        return;
      } if (this.simulation.heatExchanger === undefined && this.activatedRoute.snapshot.params.type) {
        // new simu
        this.entity.newSimulation(this.activatedRoute.snapshot.params.type);
        this.simulation = this.entity.simulation$.getValue();
      }

      this.simulationBasicColumns = this.simulation.heatExchanger.getColumnDefs('simulationBasicColumns');
      this.simulationDetailColumns = this.simulation.heatExchanger.getColumnDefs('simulationDetailColumns');
      this.widthTitle = this.arrWidthTitle[this.simulation.RADIATOR_TYPE];
      this.setColumns();
    }));

    this.combobox.getData(['2', '3', '9', '10', '12']).subscribe((res: any) => {
      this.combo2 = res.CMBMap['2'] ? res.CMBMap['2'] : [];
      this.combo3 = res.CMBMap['3'] ? res.CMBMap['3'] : [];
      this.combo9 = res.CMBMap['9'] ? res.CMBMap['9'] : [];
      this.combo10 = res.CMBMap['10'] ? res.CMBMap['10'] : [];
      this.combo12 = res.CMBMap['12'] ? res.CMBMap['12'] : [];
    });

    this.router.events.subscribe(e => {
      if (e instanceof NavigationStart) {
        if (this.router.url.indexOf('/heat-exchanger/simulation/cond') === -1
          && this.router.url !== '/heat-exchanger/simulation/result'
          && this.router.url !== '/heat-exchanger/simulation/select')
        {
          window.localStorage.setItem('clearDataApiSimuData', 'clearData');
        } else {
          window.localStorage.removeItem('clearDataApiSimuData');
        }
      }
    });
  }

  ngOnInit(): void {
    this.loading = true;
    this.showRadiatorTypeName();
    this.setRowData();
  }

  ngAfterViewInit(): void {
    this.rowData.map(v => {
      this.arrTitleRowData.push(v.title);
    });

    if (window.localStorage.getItem('clearDataApiSimuData') === 'clearData') {
      this.entity.newSimulation(this.activatedRoute.snapshot.params.type);
      this.simulation = this.entity.simulation$.getValue();
      window.localStorage.removeItem('clearDataApiSimuData');
    }

    setTimeout(() => {
      this.updateRowdata();
      this.loading = false;
    }, 0);
  }

  ngOnDestroy() {
    this.subsc.unsubscribe();
  }

  formatNumber(value: number): string | 0 {
    if (value === undefined) { return ''; }
    if (typeof value == 'number') {
      if (value == 0) {
        return 0;
      } else if (value < 0.001) {
        return value.toExponential(4);
      } else {
        return value.toFixed(5).replace(/\.00000$/, '').replace(/0000$/, '').replace(/000$/, '').replace(/00$/, '');
      }
    }
    return value;
  }

  setRowData() {
    this.rowData = [];
    this.rowData2 = [];
    this.simulationBasicColumns.map(v => {
      this.rowData.push(v.simulation)
    });

    this.simulationDetailColumns.map(v => {
      this.rowData2.push(v.simulation)
    });

    this.getDataRow = getDataRow(this.rowData, this.rowData2, this.simulation.heatExchanger.type);
  }

  showRadiatorTypeName() {
    this.radiatorTypeName = RadiatorNames[this.simulation.heatExchanger.type];
  }

  onLoading(status: boolean) {
    this.loading = status;
  }

  getModalPopupSearch(field) {
    const rowIndex = Number(window.localStorage.getItem('rowIndex'));
    const visibleIndex = Number(window.localStorage.getItem('visibleIndex'));
    const idRowSimulationCond = window.localStorage.getItem('idRowSimulationCond');
    const condition1 = rowIndex === 8 && field === 'FLUID_A_TYPE' && idRowSimulationCond;
    const condition2 = rowIndex === 9 && field === 'FLUID_B_TYPE' && idRowSimulationCond;

    if ((condition1) || (condition2)) {
      window.localStorage.setItem('popupSearch', field);
      this.ngxSmartModalService.getModal('popupSearch').open();
      this.grid1.navigateTo(rowIndex, visibleIndex, (obj) => {
        obj.target.activate();
      });
    } else {
      this.removeItemLocalStorage();
      alert('選択されていません');
    }
  }

  customKeydown(args: IGridKeydownEventArgs, grid, count = 1) {
    const target: ϴIgxGridCellComponent = args.target as ϴIgxGridCellComponent;
    const evt: KeyboardEvent = args.event as KeyboardEvent;
    const type = args.targetType;

    if (type === 'dataCell') {
      const nodeName = evt.target['nodeName'];
      const evtKey = evt.key.toLowerCase();
      let rowIndex = target.rowIndex;
      let visibleColumnIndex = target.visibleColumnIndex;

      if (evtKey === 'tab') {
        args.event.preventDefault();
        args.cancel = true;

        if (evt.shiftKey) {
          visibleColumnIndex = visibleColumnIndex < 2 ? visibleColumnIndex : visibleColumnIndex - count;
        } else {
          visibleColumnIndex = visibleColumnIndex + count;
        }

        this.checkNavigateTo(args, grid, rowIndex, visibleColumnIndex, count);
      }

      if (evtKey === 'enter') {
        if (nodeName === 'INPUT') {
          args.event.preventDefault();
          args.cancel = true;
          rowIndex = rowIndex + count;

          this.checkNavigateTo(args, grid, rowIndex, visibleColumnIndex, count);
        } else {
          this.selectedText(target);
        }
      }
    }
  }

  checkNavigateTo(event, grid, rowIndex, visibleColumnIndex, count, func = 'customKeydown') {
    if (grid.data[rowIndex]) {
      if (grid.data[rowIndex].disableAll !== true && !grid.data[rowIndex].disable.includes(`no${visibleColumnIndex}`)) {
        this.navigateToGrid(grid, rowIndex, visibleColumnIndex);
      } else {
        if (func === 'customKeydown') {
          this.customKeydown(event, grid, count + 1);
        } else {
          this.arrowKey(event, grid, count + 1);
        }
      }
    }
  }

  checkClickFluidType(event) {
    const localName = event.event.target.localName;
    const visibleIndex = event.cell.id.columnID;
    const rowIndex = event.cell.id.rowIndex;

    if (localName == 'igx-grid-cell' || localName == 'input' || localName == 'span') {
      const id = `${event.cell.grid.id}_${rowIndex}_${visibleIndex}`;
      this.fluidTypeA = Number(rowIndex) === 8 ? true : false;
      this.fluidTypeB = Number(rowIndex) === 9 ? true : false;
      window.localStorage.setItem('visibleIndex', visibleIndex);
      window.localStorage.setItem('rowIndex', rowIndex);
      window.localStorage.setItem('idRowSimulationCond', id);
    } else {
      const ukIcon = event.event.target.parentElement.getAttribute('uk-icon');

      if (ukIcon && ukIcon.includes('search')) {
        const rowIndex = window.localStorage.getItem('rowIndex');
        const fluidType = event.cell.id.rowID;

        if (trim(fluidType) === 'A流体の種類' && Number(rowIndex) === 9) {
          this.fluidTypeB = false;
          this.removeItemLocalStorage();
        }

        if (trim(fluidType) === 'B流体の種類' && Number(rowIndex) === 8) {
          this.fluidTypeA = false;
          this.removeItemLocalStorage();
        }

        return;
      }

      this.fluidTypeA = false;
      this.fluidTypeB = false;
      this.removeItemLocalStorage();
    }
  }

  removeItemLocalStorage() {
    window.localStorage.removeItem('visibleIndex');
    window.localStorage.removeItem('rowIndex');
    window.localStorage.removeItem('idRowSimulationCond');
  }

  arrowKey(event, grid, count = 1) {
    let visibleIndex = Number(grid.crudService.cell.id.columnID);
    let rowIndex = Number(grid.crudService.cell.id.rowIndex);

    if (visibleIndex !== 0) {
      event.preventDefault();
      event.cancel = true;
      if (event.keyCode === 37) {
        visibleIndex = visibleIndex < 2 ? visibleIndex : visibleIndex - count;
      }

      if (event.keyCode === 38) {
        rowIndex = rowIndex - count;
      }

      if (event.keyCode === 39) {
        visibleIndex = visibleIndex + count;
      }

      if (event.keyCode === 40) {
        rowIndex = rowIndex + count;
      }

      this.checkNavigateTo(event, grid, rowIndex, visibleIndex, count, 'arrowKey');
    }
  }

  selectedText(element) {
    const visibleIndex = Number(element.nativeElement.getAttribute('data-visibleindex'));
    const rowIndex = Number(element.nativeElement.getAttribute('data-rowindex'));

    if (visibleIndex !== 0 && rowIndex !== 0) {
      setTimeout(() => {
        element = element.nativeElement.lastElementChild;
        const tagName = element ? element.tagName.toLowerCase() : null;

        if (tagName === 'igx-input-group') {
          element.firstElementChild.firstElementChild.firstElementChild.select();
        }
      }, 100);
    }
  }

  cellEditEnter(event, rowData) {
    if (event.event?.type === 'pointerdown') {
      this.eventClick = true;
    } else {
      this.eventClick = false;
    }

    const no = `no${event.cellID.columnID}`;
    rowData.filter(obj => {
      if (obj.title === event.cellID.rowID && (obj.disableAll === true || obj.disable.includes(no))) {
        event.cancel = true;
      }
    });
  }

  checkFieldRowData2(field) {
    if (field === 'RH_A'
      || field === 'AC_A'
      || field === 'AF_A'
      || field === 'AP_A'
      || field === 'AFR_A'
      || field === 'HF_A'
      || field === 'TF_MM_A'
      || field === 'HCF_A'
      || field === 'A_A'
      || field === 'RH_B'
      || field === 'AC_B'
      || field === 'AF_B'
      || field === 'AP_B'
      || field === 'AFR_B'
      || field === 'HF_B'
      || field === 'TF_MM_B'
      || field === 'HCF_B'
      || field === 'A_B'
    ) {
      return true;
    }

    return false;
  }

  setColumns() {
    if (this.simulation.SimuInfo.length > 16) {
      this.columns = [];
      for (let i = 0; i < this.simulation.SimuInfo.length; i++) {
        this.columns.push(i + 1);
      }
    } else {
      this.columns = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16];
    }
  }

  updateRowdata(check = false) {
    if (check) {
      this.setColumns();
      this.showRadiatorTypeName();
      this.setRowData();
    }

    setTimeout(() => {
      this.columns.forEach(idx => {
        const key = 'no'+(idx);

        this.rowData = this.rowData.map(row => {
          const title = row.title;
          const field = row.field;
          const ret = Object.assign({}, row);
          const v = this.simulation.SimuInfo[idx-1];
          if (v) {
            ret[key] = this.entity.getSelectedDataSimu(field, v);

            if (field === 'WITH_FIN_A' || field === 'WITH_FIN_B') {
              ret[key] = ret[key] === '1' ? true : false;
            }
          } else {
            ret[key] = '';

            if (field === 'WITH_FIN_A' || field === 'WITH_FIN_B') {
              ret[key] = true;
            }
          }

          if (field === 'CALCULATION_TYPE') {
            if (ret[key] == 1) {
              this.setDisable('push', key, this.getDataRow.arrRequiredDeat, []);
            } else {
              this.setDisable('filter', key, this.getDataRow.arrRequiredDeat, []);
            }
          }

          if (field === 'TUBE_TYPE') {
            this.changeTubeType(ret[key], title, idx);
          }

          if (field === 'WITH_FIN_A' || field === 'WITH_FIN_B') {
            this.checkboxWithFin(ret[key], row, idx);
          }

          if (idx == this.columns.length) {
            this.grid1.updateRow(row, title);
          }

          return ret;
        });
        this.rowData2 = this.rowData2.map(row => {
          const title = row.title;
          const field = row.field;
          const ret = Object.assign({}, row);
          const v = this.simulation.SimuInfo[idx-1];
          if (v) {
            ret[key] = this.entity.getSelectedDataSimu(field, v);
          } else {
            ret[key] = '';
          }

          if (idx == this.columns.length) {
            this.grid2.updateRow(row, title);
          }

          return ret;
        });

        this.loading = false;
      });

      this.setTitleRowData(this.rowData);
    })
  }

  setTitleRowData(rowData) {
    rowData.map(row => {
      if (!row.title) {
        row.title = this.arrTitleRowData[row.index];
      }
    });
  }

  toggleDetail() {
    this.showDetail = !this.showDetail;
  }

  loadJf(c) {
    this.api.simulationCondClipboad.currentSelectNo = c;
    this.ngxSmartModalService.getModal('jfcalcLoadModal').open();
    this.navigateToGrid(this.grid1, 0, 0);
  }

  pasteJf(c) {
    this.simulation.SimuInfo[c-1] = JSON.parse(JSON.stringify(this.simulation.SimuInfo[this.api.simulationCondClipboad.pasteFrom-1]));
    this.simulation.SimuInfo[c-1].KOUMOKU = c;

    if (!this.simulation.SIMULATE_DATA_NAME) {
      this.simulation.SimuInfoOriginal[c-1] = JSON.parse(JSON.stringify(this.simulation.SimuInfoOriginal[this.api.simulationCondClipboad.pasteFrom-1]));
      this.simulation.SimuInfoOriginal[c-1].KOUMOKU = c;
    }
    this.api.simulationCondClipboad.selectedColumn[c] = false;
    this.rowData = this.rowData.map(row => {
      if (this.dirtyCheck(row, 'no'+this.api.simulationCondClipboad.pasteFrom)) {
        row.dirtys.push('no'+c);
      }
      return row;
    });
    this.rowData2 = this.rowData2.map(row => {
      if (this.dirtyCheck(row, 'no'+this.api.simulationCondClipboad.pasteFrom)) {
        row.dirtys.push('no'+c);
      }
      return row;
    });
    this.updateRowdata();
    this.navigateToGrid(this.grid1, 0, 0);
  }

  navigateToGrid(grid, rowIndex, visibleColIndex) {
    grid.navigateTo(rowIndex, visibleColIndex, (obj) => {
      obj.target.activate();
      grid.markForCheck();
      this.selectedText(obj.target);
    });
  }

  clearJf(c) {
    if (confirm('このデータを削除してもよいですか？')) {
      if (this.api.simulationCondClipboad.pasteFrom == c) {
        this.api.simulationCondClipboad.pasteFrom = 0;
      }

      let simuInfo = [];
      this.simulation.SimuInfo.map((v, i) => {
        if (i !== (c-1)) {
          simuInfo[i] = v;
        }
      });
      this.simulation.SimuInfo = simuInfo;

      if (!this.simulation.SIMULATE_DATA_NAME) {
        let simuInfoOriginal = [];

        this.simulation.SimuInfoOriginal.map((v, i) => {
          if (i !== (c-1)) {
            simuInfoOriginal[i] = v;
          }
        });
        this.simulation.SimuInfoOriginal = simuInfoOriginal;
      }

      const no = `no${c}`;
      this.rowData.map((v, i) => {
        this.rowData[i].disable = v.disable.filter(item => item !== no);
      });
      this.rowData2.map((v, i) => {
        this.rowData2[i].disable = v.disable.filter(item => item !== no);
      });

      this.updateRowdata();
    }
  }

  async setSimuInfoOriginal(index, simuInfo) {
    return new Promise((resolve, reject) => {
      const param = {
        JF_DATA_NAME: simuInfo.JF_DATA_NAME,
        KOUMOKU: simuInfo.KOUMOKU,
      };

      this.simulation.readJfdata(param, this.api, false).subscribe(res => {
        this.simulation.SimuInfoOriginal[index] = res;
        resolve('done');
      }, err => {
        reject('error');
        this.loading = false;
      });
    });
  }

  setCellSelection(event, grid, action = 'onSelection') {
    this.cellSelection = [];
    const cells = grid.selectedCells;
    let count = 0;

    if (action === 'onRangeSelection') {
      count = event.columnEnd;
    } else {
      count = event.cell.selectionNode.column;
    }

    for (let i = 0; i < count; i++) {
      this.cellSelection[i] = [];
    }

    if (this.simulation.SimuInfo.length > 0) {
      for (const cell of cells) {
        const jfNumber = cell.selectionNode.column - 1;
        const simuInfo = this.simulation.SimuInfo[jfNumber];

        if (this.cellSelection[jfNumber] === undefined) {
          this.cellSelection[jfNumber] = [];
        }

        if(simuInfo) {
          this.cellSelection[jfNumber].push(cell.row.data);
        }
      }
    }
  }

  async revert() {
    if (validateFromSearchJfcalc()) {
      this.loading = true;
      const cells = this.cellSelection;

      for (const [index, cell] of cells.entries()) {
        const simuInfo = this.simulation.SimuInfo[index];

        if(simuInfo) {
          await this.setSimuInfoOriginal(index, simuInfo);

          for (const c of cell) {
            let origValue = this.entity.getSelectedDataSimu(c.field, this.simulation.SimuInfoOriginal[index]);
            this.entity.setSelectedDataSimu(c, simuInfo, String(origValue ?? ''), 'revert');

            // this.rowData.findIndex((element, i) => {
            //   if (element['title'] === title) {
            //     this.rowData[i].dirtys = this.rowData[i].dirtys.filter(v => v != `no${index+1}`);
            //   }
            // });
          }
        }
      }

      this.updateRowdata();
    }
  }

  cellEditExit(event, gridId) {
    let row;
    const rowIndex = event.cellID.rowIndex;
    const columnID = event.cellID.columnID;

    if (gridId == 1) {
      row = this.rowData[rowIndex];
    } else if (gridId == 2) {
      row = this.rowData2[rowIndex];
    }

    const jfNumber = columnID - 1;
    const simuInfoOriginal = this.simulation.SimuInfoOriginal[jfNumber];
    if (simuInfoOriginal != undefined) {
      const origValue = this.entity.getSelectedDataSimu(row.field, simuInfoOriginal);
      if (event.newValue == origValue) {
        row.dirtys = row.dirtys.filter(v => v != 'no' + columnID);
      }
    }

    row.dirtys.push('no' + columnID);

    const simuInfo = this.simulation.SimuInfo[jfNumber];
    if (simuInfo != undefined) {
      this.entity.setSelectedDataSimu(row, simuInfo, event.newValue);
    }

    if (row.field === 'CALCULATION_TYPE') {
      const no = `no${columnID}`;

      if (row[no] == 1) {
        this.setDisable('push', no, this.getDataRow.arrRequiredDeat, []);
      } else {
        this.setDisable('filter', no, this.getDataRow.arrRequiredDeat, []);
      }
    }
  }

  dirtyPhysicalGas(value) {
    const rowIndex = value.rowIndex;
    const visibleIndex = value.visibleIndex;
    const name = value.name;
    const columnKey = `no${visibleIndex}`;
    let row = this.rowData[rowIndex];

    for (let i = 1; i <= this.columns.length; i++) {
      if (visibleIndex === i) {
        row['no'+i] = name;
        break;
      }
    }

    this.dirtyCheck(row, columnKey);
    this.grid1.updateRow(row, row.title);
  }

  dirtyCheck(rowData: any, columnKey: string) {
    return rowData.dirtys.includes(columnKey);
  }

  titleRow(rowData: any, columnKey: string) {
    return rowData.title == 'JF値データ';
  }

  dataPasted(processedData: any, grid, gridNumber) {
    this.updateRecords(processedData, grid, gridNumber);
  }

  updateRecords(processedData: any[], grid: IgxGridComponent, gridNumber: number) {
    if (grid == null) {return;}
    const cell = grid.selectedCells[0];
    const pk = grid.primaryKey;
    if (!cell) {
      return;
    }
    const rowIndex = cell.row.index;
    // const rowPkValue = cell.row.rowData[pk];
    const cellIndex = cell.column.visibleIndex;
    const columns = grid.visibleColumns;
    let index = 0;
    const updatedRecsPK = [];
    for (const curentDataRow of processedData) {
      const rowData: any = {};
      const dataRec = grid.data[rowIndex + index];
      const rowPkValue = dataRec ? dataRec[pk] : grid.data.length + 1;
      rowData[pk] = rowPkValue;
      let skip = 0;
      for (let j = 0; j < columns.length; j++) {
        const colKey = columns[j].field;
        if (colKey === undefined) {
          skip++;
          continue;
        }
        let currentCell;
        if (j - skip >= cellIndex) {
          currentCell = curentDataRow.shift();
          if (currentCell) {
            currentCell = currentCell.trim();
          }
        }

        if (dataRec) {
          if (dataRec.disableAll || dataRec.disable.includes('no'+j) || colKey === 'title') {
            rowData[colKey] = dataRec[colKey];
          } else {
            rowData[colKey] = currentCell ? currentCell : dataRec[colKey];
          }
        }
        // rowData[colKey] = currentCell || (!!dataRec ? dataRec[colKey] : null);
      }
      if (!dataRec) {
        // no rec to update, add instead
        // rowData[pk] = rowPkValue;
        // rowData['YN'] = false;
        // grid.addRow(rowData);
      } else {
        grid.updateRow(rowData, rowPkValue);
        for(let columnID=0; columnID<columns.length; columnID++) {
          rowIndex + index;
          const ev = {
            cellID: {
              rowIndex: rowIndex + index,
              rowID: rowPkValue,
              columnID: columnID + 1,
            },
            newValue: rowData['no'+(columnID+1)],
          };
          this.cellEditExit(ev, gridNumber);
        }
        grid.cdr.detectChanges();
        updatedRecsPK.push(rowPkValue);
      }
      index++;
    }
  }

  selectColumn(target) {
    this.api.simulationCondClipboad.pasteFrom = 0;
    this.api.simulationCondClipboad.selectedColumn.forEach((v, idx) => {
      if (idx == target) {
        if (this.simulation.SimuInfo[target-1]) {
          this.api.simulationCondClipboad.pasteFrom = target;
          this.api.simulationCondClipboad.selectedColumn[idx] = true;
        }
        return;
      }
      this.api.simulationCondClipboad.selectedColumn[idx] = false;
    });
  }

  addTab() {
    this.columns.push(this.columns.length + 1);
    this.rowData.map(row => {
      if (row.field === 'WITH_FIN_A' || row.field === 'WITH_FIN_B') {
        return row['no'+this.columns.length] = true;
      } else {
        return row['no'+this.columns.length] = '';
      }
    });
    this.rowData2.map(row => {
      return row['no'+this.columns.length] = '';
    });
  }

  changeTubeType(e, title, index) {
    const no = `no${index}`;
    let row2push = [];
    let row2filter = [];

    if (e === '100') {
      row2push = this.getDataRow.arrChangeTubeType[1];
      row2filter = this.getDataRow.arrChangeTubeType[0];
    } else if (e === '200') {
      row2push = this.getDataRow.arrChangeTubeType[0];
      row2filter = this.getDataRow.arrChangeTubeType[1];
    } else {
      row2filter = this.getDataRow.arrChangeTubeType[2];
    }

    this.setDisable('push', no, [], row2push);
    this.setDisable('filter', no, [], row2filter);

    const i = this.getDataRow.indexTubeType;
    let rowData = this.rowData[i];
    rowData[no] = e;
    this.grid1.updateRow(rowData, title);

    const simuInfo = this.simulation.SimuInfo[index-1];
    if (simuInfo) {
      this.entity.setSelectedDataSimu(rowData, simuInfo, e);
    }
  }

  checkboxWithFin(checked, row, index) {
    const no = `no${index}`;
    let row1 = [];
    let row2 = [];
    let i;

    if (row.field === 'WITH_FIN_A') {
      row1 = this.getDataRow.arrCheckboxWithFinA[0];
      row2 = this.getDataRow.arrCheckboxWithFinA[1];
      i = this.getDataRow.indexWithFinA;
    } else {
      row1 = this.getDataRow.arrCheckboxWithFinB[0];
      row2 = this.getDataRow.arrCheckboxWithFinB[1];
      i = this.getDataRow.indexWithFinB;
    }

    if (!checked) {
      this.setDisable('push', no, row1, row2);
    } else {
      this.setDisable('filter', no, row1, row2);
    }

    let rowData = this.rowData[i];
    rowData[no] = checked;
    this.grid1.updateRow(rowData, row.title);

    const simuInfo = this.simulation.SimuInfo[index-1];
    if (simuInfo) {
      checked = checked ? '1': '0';
      this.entity.setSelectedDataSimu(rowData, simuInfo, checked);
    }
  }

  setDisable(action, no, rowData1, rowData2) {
    if (action === 'push') {
      rowData1.map(v => {
        this.rowData[v].disable.push(no);
      });

      rowData2.map(v => {
        this.rowData2[v].disable.push(no);
      });
    } else {
      rowData1.map(v => {
        this.rowData[v].disable = this.rowData[v].disable.filter(item => item !== no);
      });

      rowData2.map(v => {
        this.rowData2[v].disable = this.rowData2[v].disable.filter(item => item !== no);
      });
    }
  }

  handleKeyUpBoardEvent(event: KeyboardEvent, grid) {
    const key = event.key;

    if ((key === 'Delete' || key === 'Backspace') && event.target['localName'] !== 'input') {
      this.deleteSelectedCell(grid);
      return;
    }

    if (key === 'ArrowUp' || key === 'ArrowDown' || key === 'ArrowLeft' || key === 'ArrowRight') {
      this.arrowKey(event, grid);
      return;
    }
  }

  handleKeyDownBoardEvent(event: KeyboardEvent, grid, table = 1) {
    const key = event.key;

    this.handleKeydown(event, grid);

    if(key == 'Tab' && table == 1) {
      if(event.shiftKey) {
        this.onShiftTab(event);
      } else {
        this.onTab(event);
      }

      return;
    }
  }

  handleKeydown(event, grid) {
    const cell = this.cellSelected;

    if (cell && !cell.editMode
      && cell.selectionNode.column !== 0
      && cell.row.data.disableAll !== true
      && !cell.row.data.disable.includes(cell._columnField)
    ) {
      getDataHandleKeyDown(event, grid, cell);
    }
  }

  onActiveNodeChange(args: IActiveNodeChangeEventArgs, grid) {
    const column = grid.getColumnByVisibleIndex(args.column);

    if (column) {
      const cell = grid.getCellByColumn(args.row, column.field);
      this.cellSelected = cell;

      if (cell && this.eventClick) {
        cell.editMode = false;
      }
    }
  }

  onShiftTab(e) {
    if (e.target.nodeName !== 'INPUT') {
      document.getElementById('add-column').focus();
    }
  }

  onTab(e) {
    if (e.target.nodeName !== 'INPUT') {
      if (!this.showDetail) {
        document.getElementById('detailed1').focus();
      } else {
        document.getElementById('detailed2').focus();
      }
    }
  }

  deleteSelectedCell(grid) {
    if (grid == null) { return; }
    const cells = grid.selectedCells;

    cells.forEach(v => {
      const data = v.row.data;

      if (data.disableAll !== true && !data.disable.includes(v._columnField)) {
        const field = data.field;
        const column = v.selectionNode.column;
        this.simulation.SimuInfo[column - 1][field] = '';

        v.update('');
      }
    })
  }

  getUnitPdf(data) {
    if (data.field === 'FLOW_RATE_A') {
      return this.combo10.find(item => item.CODE === this.api.simuUnits.FLOW_RATE_UNIT_A).VALUE;
    } else if (data.field === 'FLOW_RATE_B') {
      return this.combo10.find(item => item.CODE === this.api.simuUnits.FLOW_RATE_UNIT_B).VALUE;
    } else if (data.field === 'INLET_PRESS_A') {
      return this.combo9.find(item => item.CODE === this.api.simuUnits.INLET_PRESS_UNIT_A).VALUE;
    } else if (data.field === 'INLET_PRESS_B') {
      return this.combo9.find(item => item.CODE === this.api.simuUnits.INLET_PRESS_UNIT_B).VALUE;
    } else {
      return data.unit;
    }
  }

  convertNumber(num) {
    if ((num || num === 0) && !isNaN(Number(num))) {
      return parseFloat(Number(num).toFixed(6)) + "";
    }

    return num;
  }

  arrangeDataPdf(simuInfo, i, maxSize) {
    return new Promise((resolve, reject) => {
      const item = simuInfo.slice(i, i + maxSize);
      this.dataPdf.push(item);
      resolve('done');
    });
  }

  async exportPdf() {
    if (this.simulation.SimuInfo.length > 0) {
      const maxSize = 5;
      const simuInfo = await this.simulation.SimuInfo.filter(item => !!item);
      this.dataPdf = [];

      for (let i = 0; i < simuInfo.length; i += maxSize) {
        await this.arrangeDataPdf(simuInfo, i, maxSize);
      }

      window.frames["print_frame"].document.write('<html><head>');
      window.frames["print_frame"].document.write('<style type = "text/css">');
      window.frames["print_frame"].document.write(`
        body {
          font-family: Arial;
          font-size: 10pt;
        }
        // @media print {
        //   @page {
        //     size: landscape;
        //   }
        // }
        h1 {
          margin: 0;
        }
        table {
          border: 1px solid #dee2e6;
          border-collapse: collapse;
        }
        table th {
          background-color: #F7F7F7;
          color: #333;
          font-weight: bold;
        }
        table th, table td {
          padding: 6px;
          border: 1px solid #dee2e6;
        }
        .page-break  {
          display: block;
          page-break-before: always;
        }
        .d-flex {
          display: flex;
        }
        .mr3rem {
          margin-right: 3rem;
        }
        .text-center {
          text-align: center;
        }
        .justify-content-between {
          justify-content: space-between;
        }
        .w60px {
          width: 60px;
        }
        .w120px {
          width: 120px;
        }
        .w300px {
          width: 300px;
        }
        .w100per {
          width: 100%;
        }
        .w40per {
          width: 40%;
        }
        .w20per {
          width: 20%;
        }
        .w10per {
          width: 10%;
        }
        .mb2rem {
          margin-bottom: 2rem
        }
      `);
      window.frames["print_frame"].document.write('</style>');
      window.frames["print_frame"].document.write('</head>');
      window.frames["print_frame"].document.write('<body>');
      window.frames["print_frame"].document.write('</body>');
      window.frames["print_frame"].document.write('</html>');

      setTimeout(() => {
        window.frames["print_frame"].document.body.innerHTML = document.getElementById('pdfContent').innerHTML;
        window.frames["print_frame"].window.focus();
        window.frames["print_frame"].window.print();
      }, 500)
    }
  }
}
